@@ -4,20 +4,41 @@
 
   This program can be distributed under the terms of the GNU GPL.
   See the file COPYING.
-
-  gcc -Wall null.c `pkg-config fuse --cflags --libs` -o null
 */
 
-#define FUSE_USE_VERSION 26
+/** @file
+ *
+ * This "filesystem" provides only a single file. The mountpoint
+ * needs to be a file rather than a directory. All writes to the
+ * file will be discarded, and reading the file always returns
+ * \0.
+ *
+ * Compile with:
+ *
+ *     gcc -Wall null.c `pkg-config fuse3 --cflags --libs` -o null
+ *
+ * ## Source code ##
+ * \include passthrough_fh.c
+ */
+
+
+#define FUSE_USE_VERSION 30
+
+#include <config.h>
 
 #include <fuse.h>
+#include <fuse_lowlevel.h>
+#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <time.h>
 #include <errno.h>
 
-static int null_getattr(const char *path, struct stat *stbuf)
+static int null_getattr(const char *path, struct stat *stbuf,
+			struct fuse_file_info *fi)
 {
+	(void) fi;
+
 	if(strcmp(path, "/") != 0)
 		return -ENOENT;
 
@@ -32,9 +53,11 @@ static int null_getattr(const char *path, struct stat *stbuf)
 	return 0;
 }
 
-static int null_truncate(const char *path, off_t size)
+static int null_truncate(const char *path, off_t size,
+			 struct fuse_file_info *fi)
 {
 	(void) size;
+	(void) fi;
 
 	if(strcmp(path, "/") != 0)
 		return -ENOENT;
@@ -65,6 +88,7 @@ static int null_read(const char *path, char *buf, size_t size,
 	if (offset >= (1ULL << 32))
 		return 0;
 
+	memset(buf, 0, size);
 	return size;
 }
 
@@ -91,5 +115,27 @@ static struct fuse_operations null_oper = {
 
 int main(int argc, char *argv[])
 {
+	struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
+	struct fuse_cmdline_opts opts;
+	struct stat stbuf;
+
+	if (fuse_parse_cmdline(&args, &opts) != 0)
+		return 1;
+
+	if (!opts.mountpoint) {
+		fprintf(stderr, "missing mountpoint parameter\n");
+		return 1;
+	}
+
+	if (stat(opts.mountpoint, &stbuf) == -1) {
+		fprintf(stderr ,"failed to access mountpoint %s: %s\n",
+			opts.mountpoint, strerror(errno));
+		return 1;
+	}
+	if (!S_ISREG(stbuf.st_mode)) {
+		fprintf(stderr, "mountpoint is not a regular file\n");
+		return 1;
+	}
+
 	return fuse_main(argc, argv, &null_oper, NULL);
 }
